@@ -14,7 +14,7 @@ if os.name == "nt":
 class Service:
     def __init__ (self, name, working_dir, lockpath = None, win32service = None):
         self.name = name
-        self.working_dir = working_dir
+        self.working_dir = os.path.normpath (working_dir)
         self.lockpath = lockpath or working_dir
         self.win32service = win32service
         setproctitle and setproctitle (name)
@@ -117,25 +117,32 @@ class Service:
                 user = user,
                 executable = sys.executable,
                 serve = serve,
+                working_dir = self.working_dir,
+                pidfile = os.path.join (self.lockpath, '.pid'),
                 args = " ".join (['"{}"'.format (each) for each in args])
             )
+
+            print ('# {}.service =========================================\n'.format (name))
+            print (script)
+            print ('# end of {}.service ==================================\n'.format (name))
+
             try:
                 with open (service_script, 'w') as f:
                     f.write (script)
+
             except PermissionError:
                 service_script = os.path.join (os.getcwd (), '{}.service'.format (name))
                 with open (service_script, 'w') as f:
                     f.write (script)
-                print ('install failed, you can install manually,')
-                print ('  sudo mv {} to /etc/systemd/system'.format (service_script))
-                print ("  sudo systemctl daemon-reload")
-                print ("  sudo systemctl enable {}".format (name))
+                os.system ('sudo mv {} /etc/systemd/system'.format (service_script))
 
-            else:
-                os.system ("systemctl daemon-reload")
-                os.system ("systemctl enable {}".format (name))
-                print ("service {} installed".format (name))
+            sudo = ''
+            if os.getuid() != 0:
+                sudo = 'sudo '
+            os.system ("{}systemctl daemon-reload".format (sudo))
+            os.system ("{}systemctl enable {}".format (sudo, name))
 
+            print ("service {} installed, to start service,".format (name))
             print ("  sudo systemctl start {}".format (name))
 
         def update (self, fallback_user):
@@ -152,17 +159,23 @@ class Service:
 
 SERVICE_TEMPLATE = '''[Unit]
 Description=Skitai {name} Service
+After=syslog.target network.target
 
 [Service]
-Type=simple
+Type=forking
+PIDFile={pidfile}
+WorkingDirectory={working_dir}
 User={user}
 Group={user}
 StandardOutput=syslog+console
 StandardError=syslog+console
 Restart=on-failure
 ExecStart={executable} {serve} {args}
-ExecStop=/bin/kill -s TERM $MAINPID
 ExecReload=/bin/kill -s HUP $MAINPID
+RestartSec=2s
+TimeoutStopSec=5
+PrivateTmp=true
+KillMode=mixed
 
 [Install]
 WantedBy=multi-user.target
